# Wavefield extrapolation in (omega,x,z)-space
#
#implicit none
complex cd(96),ce(96),cf(96),q(96),aa,a,b,c,cshift
real p(96,96,12),phase,pi2,dx,dz,v,z0,x0,dt,dw,lambda,w,wov,x
integer ix,nx,iz,nz,iw,nw,it,nt,outfd,output
call initpar()
nt=12; nx=96; nz=96; dx=1.; dz=1.; pi2=2.*3.141592
v=1; lambda=nz*dz/4.; dw=v*pi2/lambda; dt=pi2/(nt*dw); 

##########################
# number of frequencies: modify here 
nw=8
#########################

call putch('n1','i',nz); call putch('n2','i',nx); call putch('n3','i',nt);
# For plotting:
call putch('ras1','i',12); call putch('ras2','i',12); call putch('gpow','i',1); 

outfd = output()
call hclose()

do iw = 1,nw {					# superimpose nw frequencies
	w =  iw*dw; wov = w/v			# frequency / velocity
	x0 = nx*dx/3.; z0 = nz*dz/3
	do ix = 1,nx {				# initial conditions for a
		x = ix*dx-x0;			# collapsing spherical wave
		phase = -wov*sqrt(z0**2+x**2)
		q(ix) = cexp(cmplx(0.,phase))
		}
	aa = (0.,1.)*dz/(4.*dx**2*wov) 		# tridiagonal matrix coef
	a = -aa;      b = 1.+2.*aa;      c = -aa
	do iz = 1,nz {				# extrapolation in depth
		do ix = 2,nx-1			# diffraction term
			cd(ix) = aa*q(ix+1) + (1.-2.*aa)*q(ix) + aa*q(ix-1)
		cd(1) = 0.;      cd(nx) = 0.
		call ctris(nx,-a,a,b,c,-c,cd,q,ce,cf)
			# Solves complex tridiagonal equations
		cshift = cexp(cmplx(0.,wov*dz))
		do ix = 1,nx			# shifting term
			q(ix) = q(ix) * cshift
		do it=1,nt {			# evolution in time
			cshift = cexp(cmplx(0.,-w*it*dt))
			do ix = 1,nx
				p(iz,ix,it) = p(iz,ix,it)+q(ix)*cshift
			}
		}
	}
do it=1,nt
	do ix=1,nx
		call rite(outfd, p(1,ix,it), nz*4)
call exit(0)
stop;		end


# This subroutine is contained in -lclass
#
subroutine ctris(n,endl,a,b,c,endr,d,q,e,f)
complex q(n),d(n),f(n),e(n),a,b,c,den,endl,endr
integer n,i
e(1) = -a/endl;		f(1) = d(1)/endl
do i = 2,n-1 {
	den = b+c*e(i-1)
	e(i) = -a/den
	f(i) = (d(i)-c*f(i-1))/den
	}
q(n) = (d(n)-c*f(n-1))/(endr+c*e(n-1))
do i = n-1,1,-1
	q(i) = e(i)*q(i+1)+f(i)
return;		end
